package com.qs.mp.user.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.admin.domain.Goods;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.TicketBoxGoods;
import com.qs.mp.admin.domain.param.IndexTicketBoxTopQueryParam;
import com.qs.mp.admin.domain.param.IndexTicketSiteTopQueryParam;
import com.qs.mp.admin.domain.param.TicketBoxGrossProfitParam;
import com.qs.mp.admin.domain.vo.*;
import com.qs.mp.admin.service.*;
import com.qs.mp.channel.domain.*;
import com.qs.mp.channel.domain.param.ChannelCommParam;
import com.qs.mp.channel.service.*;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.redis.DistributedLocker;
import com.qs.mp.common.core.redis.RedisCache;
import com.qs.mp.common.enums.*;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.RSAUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.framework.redis.RedisKey;
import com.qs.mp.framework.redis.RedisLockKey;
import com.qs.mp.framework.service.IAsyncTaskService;
import com.qs.mp.pay.domain.PayOrder;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.system.service.id.BizIdGenerator;
import com.qs.mp.user.domain.*;
import com.qs.mp.user.domain.vo.*;
import com.qs.mp.user.mapper.GroupBuyingOrderMapper;
import com.qs.mp.user.mapper.UserTicketOrderMapper;
import com.qs.mp.user.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户盲票订单 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-07
 */
@Service
public class UserGroupOrderServiceImpl extends
    ServiceImpl<GroupBuyingOrderMapper, GroupBuyingOrder> implements IUserGroupOrderService{

    protected final Logger logger = LoggerFactory.getLogger(this.getClass());


    @Value("${bind.channelId}")
    private Long bindChannelId;

    @Autowired
    private IChannelUserRelService channelUserRelService;

    @Autowired
    private IChannelOrderDetailService channelOrderDetailService;

    @Autowired
    private IUserTicketOrderItemService userTicketOrderItemService;

    @Autowired
    private ITicketService ticketService;

    @Autowired
    private IChannelService channelService;


    @Autowired
    private BizIdGenerator bizIdGenerator;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IAsyncTaskService asyncTaskService;

    @Autowired
    private DistributedLocker distributedLocker;

    @Autowired
    private ITicketBoxGoodsService ticketBoxGoodsService;

    @Autowired
    private IUserPrizeStorageService userPrizeStorageService;

    @Autowired
    private IGoodsService goodsService;

    @Autowired
    private ICouponService couponService;

    @Autowired
    private ICouponPkgService couponPkgService;


    @Autowired
    private IPromoterUserService promoterUserService;

    @Autowired
    private IGroupBuyingService groupBuyingService;

    @Autowired
    private IGroupBuyingGroupService groupBuyingGroupService;
    @Autowired
    private IUserAddrService userAddrService;
    @Autowired
    private ISysUserService userService;


    @Override
    public String submitOrder(Long userId, GroupOrderSettleVO orderSettleVO,
                              UserShareVO userShareVO) {
        UserGroupOrderServiceImpl proxy = (UserGroupOrderServiceImpl) AopContext.currentProxy();
        GroupBuying groupBuying = groupBuyingService.getById(orderSettleVO.getGroupbuyingId());
        GroupBuyingGroup groupBuyingGroup = groupBuyingGroupService.getById(orderSettleVO.getGroupId());
        //再次校验并锁次数
        checkGroup(orderSettleVO,groupBuying,groupBuyingGroup,true);
        return proxy.createOnlineOrder(userId, orderSettleVO, userShareVO,groupBuying,groupBuyingGroup);
    }

    @Override
    public boolean checkGroup(GroupOrderSettleVO orderSettleVO,GroupBuying groupBuying,GroupBuyingGroup groupBuyingGroup,boolean updateGroupNumber) {
        boolean flag =false;
        //发起拼团再次校验
        String key = RedisLockKey.build(RedisLockKey.USER_INITIATE_GROUP_LOCK,orderSettleVO.getGroupbuyingId(),orderSettleVO.getUserId());
        String groupUserKey = RedisLockKey.build(RedisLockKey.USER_JOIN_GROUP_LOCK,orderSettleVO.getGroupbuyingId(),orderSettleVO.getUserId());
        if (!distributedLocker.tryLock(key, 3, -1, TimeUnit.SECONDS)) {
            throw new ServiceException("拼团活动太火爆了,请稍后重试!");
        }
        if (!distributedLocker.tryLock(groupUserKey, 3, -1, TimeUnit.SECONDS)) {
            throw new ServiceException("拼团活动太火爆了,请稍后重试!");
        }
        if(orderSettleVO.getGroupType() == 1){
            if(ObjectUtil.isNotEmpty(redisCache.getCacheObject(key))){
                int count = redisCache.getCacheObject(key);
                if (count <= 0){
                    throw new ServiceException("您的拼团次数已经达到上限啦!");
                }
            }else {
                //设置当前用户发起拼团的最大次数
                redisCache.setCacheObject(key,groupBuying.getStartGroupTimes());
            }
            //如果用户参团次数缓存为空，并且系统设置了每个用户的允许参团的次数
            if(ObjectUtil.isEmpty(redisCache.getCacheList(groupUserKey)) && groupBuying.getPartinGroupTimes() != null){
                redisCache.setCacheObject(groupUserKey,groupBuying.getPartinGroupTimes());
            }
        }
        if(orderSettleVO.getGroupType() == 2){
            //当前团参团人数小于开团人数
            if(groupBuyingGroup.getPartinNum() < groupBuying.getGroupSize()){
                //判断用户是否还有参团名额
                int groupCount = -1;
                //设置了用户参团次数
                if(Objects.nonNull(redisCache.getCacheObject(groupUserKey))){
                    groupCount = redisCache.getCacheObject(groupUserKey);
                }
                if (groupCount == 0){
                    throw new ServiceException("您的参团次数已经达到上限啦!");
                }
            }
        }
        try {
            //锁定名额
            if(updateGroupNumber){
                updateGroupNumber(orderSettleVO,groupBuying,groupBuyingGroup);
            }
        }finally {
            distributedLocker.unlock(key);
            distributedLocker.unlock(groupUserKey);
        }
        flag = true;
        return flag;
    }

    @Override
    public boolean updateGroupNumber(GroupOrderSettleVO orderSettleVO, GroupBuying groupBuying, GroupBuyingGroup groupBuyingGroup) {
        String key = RedisLockKey.build(RedisLockKey.USER_INITIATE_GROUP_LOCK,orderSettleVO.getGroupbuyingId(),orderSettleVO.getUserId());
        String groupUserKey = RedisLockKey.build(RedisLockKey.USER_JOIN_GROUP_LOCK,orderSettleVO.getGroupbuyingId(),orderSettleVO.getUserId());
        //发起拼团，更新用户剩余拼团次数
        if(orderSettleVO.getGroupType() == 1){
            redisCache.decrement(key,1);
            redisCache.decrement(groupUserKey,1);
        }
        //参与拼团，更新用户参团次数
        if(orderSettleVO.getGroupType() == 2){
            redisCache.decrement(groupUserKey,1);
        }
        return true;
    }

    @Override
    public boolean batchCancelOrder(String boxId, List<String> orderIds) {
        return false;
    }

    @Transactional(rollbackFor = Exception.class)
    public String createOnlineOrder(Long userId, GroupOrderSettleVO orderSettleVO,
                                    UserShareVO userShareVO,GroupBuying groupBuying,GroupBuyingGroup groupBuyingGroup) {


        // 创建订单
        SysUser sysUser = userService.getById(userId);
        UserAddr userAddr =  userAddrService.getDefaultChannelAddr(userId);
        if(null != orderSettleVO.getUserAddrId()){
            userAddr = userAddrService.getById(orderSettleVO.getUserAddrId());
        }
        GroupBuyingOrder groupBuyingOrder = new GroupBuyingOrder();
        groupBuyingOrder.setOrderId(bizIdGenerator.newIdWithUidSharding(String.valueOf(userId)));
        groupBuyingOrder.setUserId(userId);
        groupBuyingOrder.setGroupbuyingTitle(groupBuying.getTitle());
        groupBuyingOrder.setUserId(userId);
        groupBuyingOrder.setAddress(userAddr.getAddr());
        groupBuyingOrder.setProvince(userAddr.getProvince());
        groupBuyingOrder.setCity(userAddr.getCity());
        groupBuyingOrder.setArea(userAddr.getArea());
        groupBuyingOrder.setCommStatus(CommStatusEnum.NO.getValue());
        groupBuyingOrder.setGroupbuyingId(groupBuying.getId());
        groupBuyingOrder.setGroupId(groupBuyingGroup.getId());
        groupBuyingOrder.setOrderAmt(orderSettleVO.getOrderAmt());
        groupBuyingOrder.setPayType(orderSettleVO.getPayType());
        groupBuyingOrder.setAppId(orderSettleVO.getAppId());
        groupBuyingOrder.setPhonenumber(sysUser.getPhonenumber());
        groupBuyingOrder.setPrizeTitle(orderSettleVO.getPrizeTitle());
        groupBuyingOrder.setPrizeQuantity(orderSettleVO.getPrizeQuantity());
        groupBuyingOrder.setPrizeType(orderSettleVO.getPrizeType());
        groupBuyingOrder.setPayAmt(orderSettleVO.getOrderAmt());

        if (orderSettleVO.getOrderAmt() == 0) {
            groupBuyingOrder.setCommStatus(CommStatusEnum.YES.getValue()); // 无需结佣，直接置为已结佣
        }

        // 查询用户的所属经销商，线上票直接查询
        ChannelUserRel channelUserRel = channelUserRelService.getOne(
                new LambdaQueryWrapper<ChannelUserRel>()
                        .eq(ChannelUserRel::getUserId, userId));
        if (null != channelUserRel) {
            groupBuyingOrder.setChannelId(channelUserRel.getChannelId());
        } else {
            // 没被绑定，看有没有分享者
            if (null != userShareVO.getSuid() && 0 != userShareVO.getSuid()) {
                if (UserShareVO.SHARE_TYPE_USER.equals(userShareVO.getType())) {
                    ChannelUserRel channelShareUserRel = channelUserRelService.getOne(
                            new LambdaQueryWrapper<ChannelUserRel>()
                                    .eq(ChannelUserRel::getUserId, userShareVO.getSuid()));
                    if (null != channelShareUserRel) {
                        groupBuyingOrder.setChannelId(channelShareUserRel.getChannelId());
                        groupBuyingOrder.setShareInfo(JSONObject.toJSONString(userShareVO));
                    }
                } else if (UserShareVO.SHARE_TYPE_SITE.equals(userShareVO.getType())) {
                    Channel channel = channelService.getOne(
                            new LambdaQueryWrapper<Channel>()
                                    .eq(Channel::getUserId, userShareVO.getSuid()));
                    if (null != channel) {
                        groupBuyingOrder.setChannelId(channel.getChannelId());
                        groupBuyingOrder.setShareInfo(JSONObject.toJSONString(userShareVO));
                    }
                }
            }
        }
        // 野生用户绑定固定的经销商
        if (groupBuyingOrder.getChannelId() == null || groupBuyingOrder.getChannelId() == 0) {
            UserShareVO tempUserShareVO = new UserShareVO();
            tempUserShareVO.setSuid(bindChannelId);
            tempUserShareVO.setType(UserShareVO.SHARE_TYPE_SITE);
            groupBuyingOrder.setChannelId(bindChannelId);
            groupBuyingOrder.setShareInfo(JSONObject.toJSONString(tempUserShareVO));
        }
        save(groupBuyingOrder);


//        if (orderSettleVO.getPayAmt() == 0) {
//            // 无需支付的，直接置为成功
//            processTicketOrder(userTicketOrder);
//            // 插入付款异步任务
//            Assert.isTrue(asyncTaskService.insertAsyncTask(AsyncTaskTypeEnum.TICKET_PAY, userTicketOrder.getOrderId()),
//                    "盲票支付，创建异步任务失败:" + userTicketOrder.getOrderId());
//        }

        //锁拼团次数
        return groupBuyingOrder.getOrderId();
    }

    @Override
    public boolean cancelOrder(String orderId) {
        return false;
    }

    @Override
    public boolean paySuccess(PayOrder payOrder) {
        return false;
    }

    @Override
    public boolean commToChannel(String orderId, Integer resource) {
        return false;
    }
}
